# Weborf
# Copyright (C) 2011-2020  Salvo "LtWorf" Tomaselli
#
# Weborf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# original code http://carnivore.it/2010/07/22/python_-_getifaddrs
# edited Salvo "LtWorf" Tomaselli <tiposchi@tiscali.it>

from ctypes import *
from socket import AF_INET, AF_INET6, AF_PACKET, inet_ntop
from sys import platform
import subprocess
import re
from typing import List, NamedTuple, Optional


def getifaddrs():
    # getifaddr structs
    class ifa_ifu_u(Union):
        _fields_ = [
            ("ifu_broadaddr", c_void_p),
            ("ifu_dstaddr",   c_void_p)
        ]

    class ifaddrs(Structure):
        _fields_ = [
            ("ifa_next",    c_void_p),
            ("ifa_name",    c_char_p),
            ("ifa_flags",   c_uint),
            ("ifa_addr",    c_void_p),
            ("ifa_netmask", c_void_p),
            ("ifa_ifu",     ifa_ifu_u),
            ("ifa_data",    c_void_p)
        ]

    # AF_UNKNOWN / generic
    class sockaddr(Structure):
        if platform.startswith("darwin") or platform.startswith("freebsd"):
            _fields_ = [
                ("sa_len",     c_uint8),
                ("sa_family",  c_uint8),
                ("sa_data",   (c_uint8 * 14))]
        else:
            _fields_ = [
                ("sa_family", c_uint16),
                ("sa_data",   (c_uint8 * 14))
            ]

    # AF_INET / IPv4
    class in_addr(Union):
        _fields_ = [
            ("s_addr", c_uint32),
        ]

    class sockaddr_in(Structure):
        _fields_ = [
            ("sin_family", c_short),
            ("sin_port",   c_ushort),
            ("sin_addr",   in_addr),
            ("sin_zero",   (c_char * 8)),  # padding
        ]

    # AF_INET6 / IPv6
    class in6_u(Union):
        _fields_ = [
            ("u6_addr8",  (c_uint8 * 16)),
            ("u6_addr16", (c_uint16 * 8)),
            ("u6_addr32", (c_uint32 * 4))
        ]

    class in6_addr(Union):
        _fields_ = [
            ("in6_u", in6_u),
        ]

    class sockaddr_in6(Structure):
        _fields_ = [
            ("sin6_family",   c_short),
            ("sin6_port",     c_ushort),
            ("sin6_flowinfo", c_uint32),
            ("sin6_addr",     in6_addr),
            ("sin6_scope_id", c_uint32),
        ]

    # AF_PACKET / Linux
    class sockaddr_ll(Structure):
        _fields_ = [
            ("sll_family",   c_uint16),
            ("sll_protocol", c_uint16),
            ("sll_ifindex",  c_uint32),
            ("sll_hatype",   c_uint16),
            ("sll_pktype",   c_uint8),
            ("sll_halen",    c_uint8),
            ("sll_addr",     (c_uint8 * 8))
        ]

    # AF_LINK / BSD|OSX
    class sockaddr_dl(Structure):
        _fields_ = [
            ("sdl_len",    c_uint8),
            ("sdl_family", c_uint8),
            ("sdl_index",  c_uint16),
            ("sdl_type",   c_uint8),
            ("sdl_nlen",   c_uint8),
            ("sdl_alen",   c_uint8),
            ("sdl_slen",   c_uint8),
            ("sdl_data",   (c_uint8 * 46))
        ]

    libc = CDLL("libc.so.6")
    ptr = c_void_p(None)
    result = libc.getifaddrs(pointer(ptr))
    if result:
        return None
    ifa = ifaddrs.from_address(ptr.value)
    result = {}

    while True:
        # name = ifa.ifa_name
        name = ifa.ifa_name.decode('ascii')  # use this for python3

        if name not in result:
            result[name] = {}

        if ifa.ifa_addr != None:
            sa = sockaddr.from_address(ifa.ifa_addr)

        if sa.sa_family not in result[name]:
            result[name][sa.sa_family] = []

        data = {}

        if sa.sa_family == AF_INET:
            if ifa.ifa_addr is not None:
                si = sockaddr_in.from_address(ifa.ifa_addr)
                data['addr'] = inet_ntop(si.sin_family, si.sin_addr)
            if ifa.ifa_netmask is not None:
                si = sockaddr_in.from_address(ifa.ifa_netmask)
                data['netmask'] = inet_ntop(si.sin_family, si.sin_addr)

        if sa.sa_family == AF_INET6:
            if ifa.ifa_addr is not None:
                si = sockaddr_in6.from_address(ifa.ifa_addr)
                data['addr'] = inet_ntop(si.sin6_family, si.sin6_addr)
                if data['addr'].startswith('fe80:'):
                    data['scope'] = si.sin6_scope_id
            if ifa.ifa_netmask is not None:
                si = sockaddr_in6.from_address(ifa.ifa_netmask)
                data['netmask'] = inet_ntop(si.sin6_family, si.sin6_addr)

        if sa.sa_family == AF_PACKET:
            if ifa.ifa_addr is not None:
                si = sockaddr_ll.from_address(ifa.ifa_addr)
                addr = ""
                for i in range(si.sll_halen):
                    addr += "%02x:" % si.sll_addr[i]
                addr = addr[:-1]
                data['addr'] = addr

        if len(data) > 0:
            result[name][sa.sa_family].append(data)

        if ifa.ifa_next:
            ifa = ifaddrs.from_address(ifa.ifa_next)
        else:
            break

    libc.freeifaddrs(ptr)
    return result


class Redirection(NamedTuple):
    '''This class represents a NAT redirection'''
    lport: int
    eport: int
    ip: str
    proto: str
    description: str

    def __str__(self) -> str:
        return '%s %d->%s:%d\t\'%s\'' % (self.proto, self.eport, self.ip, self.lport, self.description)

    def create_redirection(self) -> bool:
        '''Activates the redirection.
        Returns true if the redirection becomes active'''
        try:
            subprocess.check_call(
                ['upnpc', '-a', self.ip, str(self.lport), str(self.eport), self.proto]
            )
        except:
            return False

        for i in get_redirections():
            if i == self:
                return True
        return False

    def remove_redirection(self) -> None:
        subprocess.check_call(['upnpc', '-d', str(self.eport), self.proto])


def getaddrs(ipv6: bool=True) -> List[str]:
    '''Returns the list of the IP addresses it is possible to bind to

    ipv6: if true, ONLY ipv6 addresses will be returned (ipv4 will be mapped to ipv6)
    '''
    ifaces = getifaddrs()
    l_ipv4 = []
    l_ipv6 = []

    # Cycle the interfaces
    for iface in ifaces:

        # Cycle address family
        for family in ifaces[iface]:
            # Cycle addresses
            for addr in ifaces[iface][family]:
                if 'addr' in addr:
                    if family == AF_INET:
                        l_ipv4.append(str(addr['addr']))
                    elif family == AF_INET6:
                        l_ipv6.append(str(addr['addr']))

    if ipv6 == True:
        # Transforming ipv4 into ipv6-mapped
        for i in range(len(l_ipv4)):
            l_ipv4[i] = '::ffff:%s' % l_ipv4[i]
        return l_ipv4 + l_ipv6
    else:
        return l_ipv4


def open_nat(port: int) -> Optional[Redirection]:
    '''Tries to open a port in the nat device
    directing it to the current host.
    '''
    # Chooses the external port
    eport = port
    used_ports = {i.eport for i in get_redirections()}

    while True:
        if eport in used_ports:
            eport += 1
        else:
            break

    # Chooses the local ip
    ip = [i for i in getaddrs(False) if re.match(
        r'^10\..*', i) != None or re.match(r'^192\.168\..*', i) != None][0]

    r = Redirection(port, eport, ip, "TCP", "weborf")

    if r.create_redirection():
        return r
    return None


def externaladdr() -> Optional[str]:
    '''Returns the public IP address.
    none in case of error'''
    out = subprocess.check_output(['external-ip']).decode('ascii').strip()

    if re.match(r'[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}', out) == None:
        return None
    return out


def can_redirect() -> bool:
    '''Returns true if upnpc is installed and NAT traversal can
    be attempted'''

    try:
        subprocess.call(['upnpc'], stdout=subprocess.DEVNULL, stderr=subprocess.STDOUT)
        return True
    except:
        return False


def get_redirections() -> List[Redirection]:
    '''Returns a list of current NAT redirections'''
    out = subprocess.check_output(['upnpc', '-l']).decode('ascii').strip().split('\n')

    redirections = []

    for i in out:
        m = re.match(
            r'[ ]*([0-9]+)[ ]+(UDP|TCP)[ ]+([0-9]+)->([0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}):([0-9]+)[ ]+\'(.*)\' \'\'', i)
        if m is not None:
            redirect = m.groups()
            r = Redirection(
                int(redirect[4]),
                int(redirect[2]),
                redirect[3],
                redirect[1],
                redirect[5],
            )
            redirections.append(r)
    return redirections


def printifconfig() -> None:
    ifaces = getifaddrs()
    for iface in ifaces:
        print(iface)
        for family in ifaces[iface]:
            print("\t%s" %
                  {AF_INET: 'IPv4', AF_INET6: 'IPv6', AF_PACKET: 'HW'}[family])
            for addr in ifaces[iface][family]:
                for i in ['addr', 'netmask', 'scope']:
                    if i in addr:
                        print("\t\t%s %s" % (i, str(addr[i])))
                print("")



# Weborf
# Copyright (C) 2010-2024  Salvo "LtWorf" Tomaselli
#
# Weborf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# author Salvo "LtWorf" Tomaselli <tiposchi@tiscali.it>

import os
import subprocess
import socket
import threading
from typing import Optional
from PyQt6 import QtCore

import qweborf.nhelper as nhelper


class weborf_runner():

    def __init__(self, logfunction):
        self.logclass = logfunction
        self.logclass.logger("Software initialized")

        self.child: Optional[subprocess.Popen] = None
        self.socket = None
        self.listener = None
        self.waiter: Optional[_Waiter] = None
        self.methods = set()
        self.username: Optional[str] = None
        self.password: Optional[str] = None
        self.ipv6 = True
        self._running = False
        self.version = ''
        self.webdav = False

        self.weborf = self._test_weborf()

    def _test_weborf(self) -> bool:
        '''Tests if weborf binary is existing.
        It will return true if everything is OK
        and false otherwise.'''

        try:
            out = subprocess.check_output(["weborf", "-k"]).decode('ascii').strip().split('\n')
        except Exception as e:
            self.logclass.logger(f'Unable to check capabilities weborf.\n{e}', self.logclass.DBG_ERROR)
            return False

        try:
            capabilities = {}
            for i in out:
                k, v = i.split(':', 1)
                capabilities[k] = v

            self.version = capabilities['version']
            self.ipv6 = capabilities['ipv'] == '6'
            self.webdav = capabilities['webdav'] == 'yes'
            self.https = capabilities.get('https') == 'yes'

            self.logclass.logger('Weborf version %s found' %
                                    self.version, self.logclass.DBG_NOTICE)
            self.logclass.logger('IPv%s protocol in use' %
                                    capabilities['ipv'], self.logclass.DBG_NOTICE)
            self.logclass.logger('Has webdav support: %s' % capabilities['webdav'], self.logclass.DBG_NOTICE)

            if capabilities['embedded_auth'] == 'yes':
                self.logclass.logger('Binary compiled with embedded authentication', self.logclass.DBG_ERROR)
                return False

            return True
        except KeyError as e:
            self.logclass.logger(
                'Capability %s not supported' % e, self.logclass.DBG_ERROR)
        except Exception as e:
            self.logclass.logger(
                'Unknown exception %s' % e, self.logclass.DBG_ERROR)

        return False

    def start(self, options) -> bool:
        '''Starts weborf,
        returns True if it is correctly started'''

        if not self.weborf:
            self.logclass.logger(
                'Unable to start weborf', self.logclass.DBG_ERROR)
            return False

        if len(options['path']) == 0:
            self.logclass.logger('Path not specified', self.logclass.DBG_ERROR)
            return False

        self.logclass.logger("Starting weborf...")

        auth_socket = self.__create_auth_socket()
        if not self.__start_weborf(options, auth_socket):
            return False
        self.__listen_auth_socket(options)

        self.username = options['username']
        self.password = options['password']

        # Deciding which HTTP methods will be enabled
        self.methods = {'GET'}
        if options['dav']:
            self.methods.update({'OPTIONS', 'PROPFIND'})
            self.logclass.logger(
                "DAV access enabled", self.logclass.DBG_NOTICE)

            # If writing is enabled
            if options['write']:
                self.methods.update({'PUT', 'DELETE', 'COPY', 'MOVE', 'MKCOL'})
                self.logclass.logger(
                    'Writing access enabled.', self.logclass.DBG_WARNING)
        return True

    def __create_auth_socket(self):
        '''Creates a unix socket and returns the path to it'''
        self.socket = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        sockname = "/tmp/weborf_auth_socket%d-%d.socket" % (
            os.getuid(), os.getpid())

        try:
            os.remove(sockname)
        except OSError:
            pass

        self.socket.bind(sockname)
        return sockname

    def __listen_auth_socket(self, options):
        self.listener = __listener__(self.socket)
        self.listener.new_socket.connect(self.socket_cback)
        self.listener.start()

    def socket_cback(self, sock):
        '''Recieves connection requests and decides if they have to be authorized or denied'''

        data = sock.recv(4096).split(b'\r\n')

        if len(data) < 5:
            sock.send(b' ')
            sock.close()
            self.logclass.logger("DENIED: Invalid request")
            return

        uri = data[0].decode('utf-8')
        client = data[1].decode()
        method = data[2].decode()
        username = data[3].decode('utf-8')
        password = data[4].decode('utf-8')

        # Checking if the request must be allowed or denied
        allow = True
        if self.username is not None:  # Must check username and password
            allow = self.username == username and self.password == password

        if method not in self.methods:
            allow = False

        if allow:
            self.logclass.logger("%s - %s %s" % (client, method, uri))
        else:
            sock.send(b' ')
            self.logclass.logger("DENIED: %s - %s %s" % (client, method, uri))

        sock.close()

    def __start_weborf(self, options, auth_socket) -> bool:
        '''Starts a weborf in a subprocess'''

        cmdline = ["weborf", "-p", str(options['port']), "-b", str(
            options['path']), "-I", "....", "-a", auth_socket]

        if options['tar'] and (options['cert'] or options['key']):
            self.logclass.logger('Tar and HTTPS cannot be enabled at the same time', self.logclass.DBG_ERROR)
            return False

        if options['tar']:
            cmdline.append('--tar')

        if options['cert'] or options['key']:
            cmdline.extend([
                '--cert',
                options['cert'],
                '--key',
                options['key'],
            ])

        if options['ip'] != None:
            cmdline.append('-i')
            cmdline.append(options['ip'])
        self.logclass.logger(' '.join(cmdline))

        self.child = subprocess.Popen(cmdline)

        self.loglinks(options)

        # Starts thread to wait for weborf termination
        self.waiter = _Waiter(self.child)

        self.waiter.child_terminated.connect(self._child_terminated)
        self.waiter.start()
        self._running = True

        return True

    def loglinks(self, options):
        '''Prints to the log all the links that weborf is listening to'''
        if options['ip'] == None:
            addrs4 = nhelper.getaddrs(False)

            if self.ipv6:
                addrs6 = nhelper.getaddrs(True)
            else:
                addrs6 = []
        else:
            if self.ipv6:
                # address can be both ipv6 or mapped ipv4
                if '.' in options['ip']:
                    addrs6 = [options['ip']]
                    addrs4 = [options['ip'][7:]]
                else:  # Normal ipv6
                    addrs4 = []
                    addrs6 = [options['ip']]
            else:
                addrs6 = []
                addrs4 = [options['ip']]

        if options['cert'] or options['key']:
            protocol = 'https'
        else:
            protocol = 'http'

        # Output of addresses bound
        for i in addrs4:
            url = f'{protocol}://%s:%d/' % (i, options['port'])
            logentry = 'Address: <a href="%s">%s</a>' % (url, url)
            self.logclass.logger(logentry)
        for i in addrs6:
            url = f'{protocol}://[%s]:%d/' % (i, options['port'])
            logentry = 'Address: <a href="%s">%s</a>' % (url, url)
            self.logclass.logger(logentry)

    def _child_terminated(self, child, exit_code):
        '''Called when the child process is terminated
        param child is for now ignored'''
        if exit_code != 0:
            self.logclass.logger('Weborf terminated with exit code %d' %
                                 exit_code, self.logclass.DBG_ERROR)
        else:
            self.logclass.logger(
                "Termination complete", self.logclass.DBG_NOTICE)
        self._running = False
        pass

    def stop(self):
        '''Stop weborf and correlated processes.
        Will return True if it goes well
        It should be called only if start succeeded'''
        if self._running:
            self.logclass.logger(
                "Sending terminate signal and waiting for termination...")
            self.child.terminate()
        if self.socket:
            self.socket.close()
                          # Closing socket, so the accept will fail and the
                          # thread can terminate
            self.listener.stop()

        return True


class __listener__(QtCore.QThread):

    '''This class is used to listen to a socket.
    It will accept connections and send those connection
    using new_socket(PyQt_PyObject).
    '''

    new_socket = QtCore.pyqtSignal(object)

    def __init__(self, socket):
        QtCore.QThread.__init__(self)
        self.socket = socket
        self.socket.settimeout(2.0)
        self.cycle = True

    def stop(self):
        self.cycle = False

    def run(self):
        print("Ready to listen")
        self.socket.listen(1)
        while self.cycle:
            try:
                sock, addr = self.socket.accept()
                print(sock, addr)
                self.new_socket.emit(sock)
            except:
                pass


class _Waiter(QtCore.QThread):

    '''This class creates a separate thread that will wait for the
    termination of weborf, and performs a callback when it occurs.
    A more normal way to do this would have been to handle SIGCHLD but
    the use of QT libraries prevents the use of signals apparently.

    connect to child_terminated(PyQt_PyObject,PyQt_PyObject) to handle the event'''

    child_terminated = QtCore.pyqtSignal(object, object)

    def __init__(self, child) -> None:
        '''child: child process to wait
        '''
        QtCore.QThread.__init__(self)

        self.child = child

    def run(self):

        # wait termination for the child process
        t = self.child.wait()

        # Sends callback
        self.child_terminated.emit(self.child, t)



#!/usr/bin/python3
import os
print('X-Extra: Ciao\r\n', end='')
print('Content-Type: text/plain\r\n', end='')
print('\r\n', end='')

for k,v in os.environ.items():
    print(f'{k}\t{v}')

if 'CONTENT_LENGTH' in os.environ:
    print('============= POST')
    data = os.read(0, int(os.environ['CONTENT_LENGTH']))
    print(data)



#!/usr/bin/python
# -*- coding: utf-8 -*-

'''
This is an example of index file.
It adds a "." to a session variable on every refresh, shows the informations and sets a cookie.
'''

import os
import sys
from cgi_weborf import * #imports module cgi

cgi.session_start() 
cgi.setcookie("id","33")

cgi.finalize_headers() #will output the final \r\n\r\n and the content-type header

if 'nome' in cgi.SESSION:
        cgi.SESSION['nome']+="."
else:
        cgi.SESSION['nome']="."

sys.stdout.write( "<html>")
sys.stdout.write( "<body>")

sys.stdout.write( "<h1>example</h1>")
cgi.pyinfo()
sys.stdout.write(  "</body>")
sys.stdout.write(  "</html>")

cgi.savesession()



from setuptools import setup

setup(
    version='1.6',
    name='qweborf',
    author="Salvo 'LtWorf' Tomaselli",
    author_email='tiposchi@tiscali.it',
    maintainer="Salvo 'LtWorf' Tomaselli",
    maintainer_email='tiposchi@tiscali.it',
    url='https://ltworf.codeberg.page/weborf/',
    license='GPL3',
    packages=('qweborf',),
    )




#!/usr/bin/python3
# This is an example authentication script. You can modify it to make it fit your needs

# Weborf
# Copyright (C) 2019-2023  Salvo "LtWorf" Tomaselli
#
# Weborf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# author Salvo "LtWorf" Tomaselli <tiposchi@tiscali.it>

import asyncio
import os
from typing import NamedTuple
from syslog import *


class Request(NamedTuple):
    url: bytes
    addr: bytes
    method: bytes
    username: bytes
    password: bytes
    protocol: bytes

SOCKET_PATH = "/run/weborf/weborf_auth.socket"

async def main():
    try:
        os.remove(SOCKET_PATH)
    except OSError:
        pass

    l = asyncio.get_running_loop()
    server = await l.create_unix_server(Proto, SOCKET_PATH)
    await server.serve_forever()


class Proto(asyncio.Protocol):
    def allow(self):
        self.transport.close()

    def deny(self):
        self.transport.write(b' ')
        self.transport.close()

    def connection_made(self, transport):
        self.transport = transport

    def data_received(self, data: bytes) -> None:
        fields = data.split(b'\r\n')
        request = Request(*fields[:6])
        headers = {k:v for k, v in(i.split(b': ', 1) for i in fields[6:] if i)}

        # Allow all from internal network
        if request.addr.startswith(b'::ffff:10.0.'):
            self.allow()
            return

        # Require a password for some content
        if request.url.startswith(b'/movies/') or request.url.startswith(b'/music/'):
            if request.username != b'user' and request.password != b'pass':
                self.deny()
                return
            else:
                self.allow()
                return
        # Require another password for all other content
        elif request.username == b'user2' and request.password == b'pass2':
            self.allow()
            return
        self.deny()


asyncio.run(main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Weborf
# Copyright (C) 2010-2025  Salvo "LtWorf" Tomaselli
#
# Weborf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# author Salvo "LtWorf" Tomaselli <tiposchi@tiscali.it>

from PyQt6 import QtGui, QtWidgets, QtCore
import sys
import os

import qweborf.main as main
import qweborf.whelper as whelper
import qweborf.nhelper as nhelper


class qweborfForm (QtWidgets.QWidget):

    DBG_DEFAULT = 0
    DBG_WARNING = 1
    DBG_ERROR = 2
    DBG_NOTICE = 3

    def setUi(self, ui) -> None:
        self.settings = QtCore.QSettings()
        self.ui = ui
        self.weborf = whelper.weborf_runner(self)
        self.started = False

        if self.weborf.version < '0.13':
            sys.exit('Ancient version of weborf detected')

        if not self.weborf.webdav:
            self.ui.chkDav.setEnabled(False)
            self.ui.chkWrite.setEnabled(False)

        if not self.weborf.https:
            self.ui.txtKey.setEnabled(False)
            self.ui.txtCert.setEnabled(False)
            self.ui.cmdOpenCert.setEnabled(False)
            self.ui.cmdOpenKey.setEnabled(False)

        # Listing addresses
        for i in nhelper.getaddrs(self.weborf.ipv6):
            self.ui.cmbAddress.addItem(i, None)

        self.ui.chkNAT.setEnabled(nhelper.can_redirect())

        self.defaultdir = QtCore.QStandardPaths.writableLocation(
            QtCore.QStandardPaths.StandardLocation.HomeLocation)

        initialdir = self.settings.value('main/defaultdir', self.defaultdir)

        self.ui.chkDav.setChecked(self.settings.value('dav/dav', 'false') == 'true')
        self.ui.chkWrite.setChecked(self.settings.value('dav/dav_write', 'false') == 'true')
        self.ui.chkEnableAuth.setChecked(self.settings.value('auth/auth', 'false') == 'true')
        self.ui.txtPassword.setText(self.settings.value('auth/password', ''))
        self.ui.txtUsername.setText(self.settings.value('auth/username', ''))

        self.ui.chkTar.setChecked(self.settings.value('tar/tar', 'false') == 'true')

        self.ui.txtCert.setText(self.settings.value('ssl/cert', ''))
        self.ui.txtKey.setText(self.settings.value('ssl/key', ''))

        self.ui.chkNAT.setChecked(self.settings.value('net/nat', 'false') == 'true')
        self.ui.cmbAddress.setCurrentIndex(int(self.settings.value('net/address', 0)))
        self.ui.spinPort.setValue(int(self.settings.value('net/port', 8080)))

        if len(sys.argv) > 1 and os.path.exists(sys.argv[1]):
            initialdir = sys.argv[1]

        self.ui.txtPath.setText(initialdir)
        self.stop_sharing()

    def logger(self, data, level=DBG_DEFAULT):
        '''logs an entry, showing it in the GUI'''
        if level == self.DBG_WARNING:
            data = '<font color="orange"><strong>WARNING</strong></font>: %s' % data
        elif level == self.DBG_ERROR:
            data = '<font color="red"><strong>ERROR</strong></font>: %s' % data
        elif level == self.DBG_NOTICE:
            data = '<font color="green"><strong>NOTICE</strong></font>: %s' % data
        self.ui.txtLog.moveCursor(QtGui.QTextCursor.MoveOperation.End)
        self.ui.txtLog.insertHtml(data + '<br>')
        self.ui.txtLog.moveCursor(QtGui.QTextCursor.MoveOperation.End)

    def setDefaultValues(self):
        '''Sets default values into the form GUI. It has to be
        called after the form has been initialized'''
        pass

    def stop_sharing(self):
        self.weborf.stop()
        self.ui.cmdStart.setVisible(True)
        self.ui.cmdStop.setVisible(False)
        self.ui.tabWidget.setVisible(True)
        self.ui.LogLayout.setVisible(False)
        self.started = False

    def save_settings(self):
        self.settings.setValue('main/defaultdir', self.ui.txtPath.text())
        self.settings.setValue('dav/dav', self.ui.chkDav.isChecked())
        self.settings.setValue('dav/dav_write', self.ui.chkWrite.isChecked())
        self.settings.setValue('auth/auth', self.ui.chkEnableAuth.isChecked())
        self.settings.setValue('auth/password', self.ui.txtPassword.text())
        self.settings.setValue('auth/username', self.ui.txtUsername.text())
        self.settings.setValue('tar/tar', self.ui.chkTar.isChecked())
        self.settings.setValue('ssl/cert', self.ui.txtCert.text())
        self.settings.setValue('ssl/key', self.ui.txtKey.text())
        self.settings.setValue('net/nat', self.ui.chkNAT.isChecked())
        self.settings.setValue('net/address', self.ui.cmbAddress.currentIndex())
        self.settings.setValue('net/port', self.ui.spinPort.value())

    def about(self):

        self.logger('<hr><strong>Qweborf 1.6</strong>')
        self.logger('This program comes with ABSOLUTELY NO WARRANTY.'
                    ' This is free software, and you are welcome to redistribute it under certain conditions.'
                    ' For details see the <a href="http://www.gnu.org/licenses/gpl.html">GPLv3 Licese</a>.')
        self.logger(
            '<a href="https://ltworf.codeberg.page/weborf/">Homepage</a>')
        self.logger(
            'Salvo \'LtWorf\' Tomaselli <a href="mailto:tiposchi@tiscali.it">&lt;tiposchi@tiscali.it&gt;</a>')

    def terminate(self):
        if self.started:
            self.stop_sharing()
        sys.exit(0)

    def start_sharing(self):

        options = {}  # Dictionary with the chosen options

        options['path'] = self.ui.txtPath.text()

        if self.ui.chkEnableAuth.isChecked():
            options['username'] = self.ui.txtUsername.text()
            options['password'] = self.ui.txtPassword.text()
        else:
            options['username'] = None
            options['password'] = None

        if self.ui.txtKey.text() or self.ui.txtCert.text():
            options['key'] = self.ui.txtKey.text()
            options['cert'] = self.ui.txtCert.text()
        else:
            options['key'] = None
            options['cert'] = None

        options['port'] = self.ui.spinPort.value()

        options['dav'] = self.ui.chkDav.isChecked()
        if self.ui.chkDav.isChecked():
            options['write'] = self.ui.chkWrite.isChecked()
        else:
            options['write'] = False

        options['tar'] = self.ui.chkTar.isChecked()

        if self.ui.cmbAddress.currentIndex() == 0:
            options['ip'] = None
        else:
            options['ip'] = str(self.ui.cmbAddress.currentText())

        if self.weborf.start(options):
            self.ui.cmdStart.setVisible(False)
            self.ui.cmdStop.setVisible(True)
            self.ui.tabWidget.setVisible(False)
            self.ui.LogLayout.setVisible(True)
            self.started = True

        if self.ui.chkNAT.isChecked() and self.started==True:
            self.logger('Trying to use UPnP to open a redirection in the NAT device. Please wait...')
            QtCore.QCoreApplication.processEvents()
            external_addr = nhelper.externaladdr()
            QtCore.QCoreApplication.processEvents()
            self.logger('Public IP address %s' % str(external_addr))
            QtCore.QCoreApplication.processEvents()
            if external_addr is not None:
                redirection=nhelper.open_nat(options['port'])
                if redirection is not None:
                    self.redirection=redirection
                    url='http://%s:%d/' % (external_addr,redirection.eport)
                    logentry='Public address: <a href="%s">%s</a>' % (url,url)
                    self.logger(logentry)
                else:
                    self.logger('Could not create NAT route')
            else:
                self.logger('Could not find the external IP address')

    def select_cert(self) -> None:
        fname = QtWidgets.QFileDialog.getOpenFileName(
            self,
            'Certificate',
            self.ui.txtCert.text(),
            filter='PEM certificates(*.pem);;All files(*)',
        )[0]
        if fname:
            self.ui.txtCert.setText(fname)


    def select_key(self) -> None:
        fname = QtWidgets.QFileDialog.getOpenFileName(
            self,
            'Certificate',
            self.ui.txtKey.text(),
        )[0]
        if fname:
            self.ui.txtKey.setText(fname)

    def select_path(self) -> None:
        current = self.ui.txtPath.text()
        if len(current) == 0:
            current = self.defaultdir
        dirname = QtWidgets.QFileDialog.getExistingDirectory(
            self,
            'Directory to share',
            current,
            QtWidgets.QFileDialog.Option.ShowDirsOnly
        )
        if len(dirname) > 0:
            self.ui.txtPath.setText(dirname)


def q_main() -> None:
    import sys
    app = QtWidgets.QApplication(sys.argv)
    app.setOrganizationName('weborf')
    app.setApplicationName('qweborf')
    app.setOrganizationDomain("org.ltworf")
    Form = qweborfForm()
    ui = main.Ui_Form()
    ui.setupUi(Form)
    Form.setUi(ui)
    Form.show()
    res = app.exec()
    Form.terminate()
    sys.exit(res)

if __name__ == "__main__":
    q_main()
